#!/usr/bin/env python3
import requests, json
from requests.auth import HTTPBasicAuth
import datetime, os, logging, shutil
import argparse, configparser

# USAGE
# This script delete pastell documents listed in json file.
# json file is a dict with this mandatory properties id_e and document inside a docs array :
# {"docs": [{"id_e": "1", "document": "9uTg9B4"},...]}
# option -s or --source => path to json file containing list of documents to delete
# option -e or --env => to select environnement execution (prod or preprod). default is preprod
# option -l or --limit => define a limit of documents to delete. Default is 0 = no limit
#
# ./pastell_delete.py --env=preprod|prod [--limit=10] --source=myfile.json
#

parser = argparse.ArgumentParser()
parser.add_argument("-e", "--env", type=str, help="Sélection de l'environnement", choices=['preprod','prod'], required=True)
parser.add_argument("-l", "--limit", type=int, help="Limite le nombre de suppression au nombre défini")
parser.add_argument("-s", "--source", required=True, type=str, help="Chemin vers le fichier json contenant les id_d des documents à supprimer")
args = parser.parse_args()

ENV = args.env
LIMIT = args.limit
FILE_DOCS_TO_DELETE = args.source
LOG_FILE = "/tmp/delete.log"

user_config_dir = os.path.expanduser("~")
user_config = user_config_dir + "/.pastell-admin"
if not os.path.isfile(user_config):
  os.makedirs(user_config_dir, exist_ok=True)
  shutil.copyfile("./pastell-admin.dist", user_config)

config = configparser.ConfigParser()
config.optionxform = lambda option: option
config.read(user_config)

SERVER = config[ENV]['server']
user = config[ENV]['login']
pwd = config[ENV]['password']

#logs
FORMAT = '%(asctime)s — %(name)s — %(levelname)s — %(message)s'
logging.basicConfig(filename=LOG_FILE, format=FORMAT)
logger = logging.getLogger('pastell-delete %s' % ENV)
logger.setLevel(logging.DEBUG) # better to have too much log than not enough

start = datetime.datetime.now()
logger.info("Début de la suppression des documents à %s" % start)

print("API PASTELL : Suppression des documents en %s" % ENV)

with open(FILE_DOCS_TO_DELETE, 'r', encoding='UTF8') as f:
    data = json.load(f)
docs = data["docs"]
logger.info("liste des documents à supprimer chargée %s" % FILE_DOCS_TO_DELETE)

TASK = 1
for doc in docs:
  if not LIMIT or TASK <= LIMIT:
    TASK +=1
    url1 = "https://%s/api/v2/entite/%s/document/%s/action/fatal-error" % (SERVER, doc['id_e'], doc["document"])
    request1 = requests.post(url1, auth=HTTPBasicAuth(user, pwd))
    if request1.status_code in [200, 201, 400, 403]:
        result1 = json.loads(request1.text)
        if "result" in result1:
            if result1["result"] == True:
                print("Suppression document %s/%s" % (doc['id_e'], doc["document"]))
                url = "https://%s/api/v2/entite/%s/document/%s/action/supression" % (SERVER, doc['id_e'], doc["document"])
                request = requests.post(url, auth=HTTPBasicAuth(user, pwd))
                print(request.status_code)
                print(request.text)
                if request.status_code in [200, 201, 400, 403]:
                    try:
                        result = json.loads(request.text)
                        keys = result.keys()
                        if "status" in keys:
                            if result["status"] == "error":
                                logger.warning("Erreur suppression du document %s/%s : %s" % (doc['id_e'],doc["document"],result["error-message"]))

                        elif "result" in keys:
                            if result["result"] == True:
                                logger.info("document %s/%s %s supprimé avec succès" % (doc['id_e'],doc["document"], result["message"]))

                    except ValueError as err:
                        logger.warning("Erreur %s" % err)
                else:
                    logger.warning("Erreur de requête : %s" % url)


end = datetime.datetime.now()
delta = end - start
msg = f"script exécuté en {delta.seconds // 60} minutes et {delta.seconds % 60} secondes"
print(msg)
logger.info(msg)



#!/usr/bin/env python3
import csv
from datetime import datetime
import argparse
from .api.client import Session, Connector
from .shared.configuration import getConfiguration

# USAGE
# This script list pastell all connector instances and export result in a csv file.
# option -e or --env => to select environnement execution (prod or preprod). default is preprod
# option -i or --ide => pastell entity to filter. eg : 1
#
# ./pastell_connector_instances.py --env=preprod --ide=1
#
#

def script_wrapper():
  xstart = datetime.now()
  parser = argparse.ArgumentParser()
  parser.add_argument("-e", "--env", type=str, help="Sélection de l'environnement", choices=['preprod','prod'], required=True)
  parser.add_argument("-o", "--org", help="id_e de l'entité", default='all')
  parser.add_argument("-c", "--csv", help="Fichier csv en sortie. /tmp/instances.csv par défaut")
  args = parser.parse_args()

  instances(env=args.env, org=args.org, csv=args.csv)

  xend = datetime.now()
  delta = xend - xstart
  msg = f"info : script exécuté en {delta.seconds // 60} minutes et {delta.seconds % 60} secondes"
  print(msg)

def instances(env, org='all', csv="/tmp/instances.csv" ):
  #Get parameters from config file ~/.pastell-admin
  config = getConfiguration()

  server = config[env]['server']
  user = config[env]['login']
  pwd = config[env]['password']
  session = Session(server,  user, pwd)


  fields = ['id_ce', 'id_e', 'libelle', 'id_connecteur', 'type']
  csv_list = []
  connectors = Connector(session).getAll(org).result
  for instance in connectors:
    data = []
    for field in fields:
      data.append(instance[field])
    csv_list.append(data)

  writeResults(csv_list, csv, fields)

def writeResults(csv_list, output, fields):
  with open(output, 'w', encoding='UTF8', newline='') as f:
    writer = csv.writer(f)
    # write the header
    writer.writerow(fields)
    # write multiple rows
    writer.writerows(csv_list)

if __name__ == '__main__':
  script_wrapper()



#Get parameters from config file ~/.pastell-admin
import os, shutil, configparser

def getConfiguration():
    user_config_dir = os.path.expanduser("~")
    user_config = user_config_dir + "/.pastell-admin"
    if not os.path.isfile(user_config):
        os.makedirs(user_config_dir, exist_ok=True)
        shutil.copyfile("./pastell-admin.dist", user_config)

    config = configparser.ConfigParser()
    config.optionxform = lambda option: option
    config.read(user_config)
    return config


#!/usr/bin/env python3
import  json, re
import os
from datetime import datetime
import argparse
from .api.client import Session, Entity, Association, Connector
from .shared.configuration import getConfiguration

"""
Ce script permet de gérer des connecteurs pour une instance Pastell:

     - statut d'un connecteur (absence, présence, doublons) unitaire ou en masse
     - création de connecteurs et association unitaire ou en masse
     - Mise à jour d'un connecteur unitaire ou en masse
     - Mise à jour et création si nécessaire d'un connecteur unitaire ou en masse
     - Désassociation d'un connecteur unitaire ou en masse
     - Suppression d'un connecteur unitaire ou en masse

  Args:
    option -e or --env => to select environnement execution (prod or preprod). default is preprod
    option -a or --action => Mandatory : choose between [create, update, append, dissociate, delete, status]
    option -o or --org => pastell entity to filter. eg : 1. Optional
    option -f or --flux => Type de dossier (flux). Exemple autres-studio-sans-tdt; Mandatory for action (dissociate and delete without source file )
    option -s or --source => fichier json contenant la définition du connecteur ou la liste des connecteurs à supprimer [{"id_e":"", "id_ce"},]
    option -l or --libelle => Libellé du connecteur. Mandatory for status, dissociate, delete
    option -t or --type => type de connecteur (eg. GED). Mandatory for dissociate and delete
    option -i or --id_connector => id_connecteur (eg. depot-pastell). Mandatory for dissociate and delete
    option -d or --details => A utiliser avec l'action status. Pour afficher les id_e des entités concernées.
    option -r or --recursive => : précise si les entités filles sont également concernées. A associer avec --org=


  Usage :
    ./pastell_connector.py --env=preprod --action=status --libelle=ged-megalis-opendata-sans-tdt --details
    ./pastell_connector.py --env=preprod --org=7 --action=create --flux=autres-studio-sans-tdt --source=connectors/opendata-sans-tdt.json
    ./pastell_connector.py --env=preprod --org=7 --action=update --flux=autres-studio-sans-tdt --source=connectors/opendata-sans-tdt.json
    ./pastell_connector.py --env=preprod --org=7 --recursive --action=append --flux=autres-studio-sans-tdt --source=connectors/opendata-sans-tdt.json
    ./pastell_connector.py --env=preprod --org=7 --action=dissociate --flux=autres-studio-sans-tdt --type=GED --libelle=ged-megalis-opendata-sans-tdt --id_connector=depot-pastell
    ./pastell_connector.py --env=preprod --org=7 --action=delete --type=GED --libelle=ged-megalis-opendata-sans-tdt --id_connector=depot-pastell# ./pastell_connector.py --env=preprod --org=7 --action=delete --source=connectors/connectors_to_delete.json

"""

def variables_substitution(o):
  result = o.copy()
  for key, val in o.items():
    variable = re.findall('{{(.*?)}}', val)
    if variable and os.getenv(variable[0]):
      result[key] = os.getenv(variable[0])
  return result

def taskConnectorFor(session, scope, action, connector_template, delete_list, flux_associe, lib_connector, type_connector, id_connector, details, recursive, scheduler):
  global connector
  if scope == 'all':
    # Get all entities from Pastell API
    request = Entity(session).getAll()
    if request.success:
      entities = request.result
    else:
      print(f"error : {request.result}")
  elif recursive:
    request = Entity(session).getFilles(scope)
    if request.success:
      entities = request.result
    else:
      print(f"error : {request.result}")
  else:
    # Create list with the id_e passed in parameter
    entities = [{"id_e": scope }]
  # DELETE action WITH LIST
  if action == 'delete' and delete_list:
    for connector in delete_list:
      deleteConnector(connector["id_e"],connector["id_ce"])
  else:
    #Get complete list of connectors with id_connecteur type (eg depot-pastell) for the scope (one id_e if not recursive option or all id_e)
    if recursive and len(entities) > 1:
      scope = 'all'
    request = Connector(session).getAll(scope, id_connecteur=id_connector, outputFormat='dict')
    connectors = {}
    if request.success:
      connectors = request.result

    #Filter entities list for create action and delete action
    # create action for entities without the lib_connector
    # update action for entities with the lib_connector
    if action in ['status', 'create', 'update', 'delete']:
      #select existing id_e with this connector
      existing_same_connectors = [ide for ide, v in connectors.items() if lib_connector in v.keys()]
      doublons = [ide for ide, v in connectors.items() if lib_connector in v.keys() and len(v[lib_connector]) > 1 ]
      if action in ('create', 'status'):
        print(f"info : {len(existing_same_connectors)} entité(s) dispose(nt) déjà du connecteur {lib_connector}")
        if len(doublons) > 0:
          print(f"warning : connecteur {lib_connector} en doublons pour {len(doublons)} entité(s)")
        #Filter entities to include only entities with not this connector
        entities = [{"id_e": entity["id_e"]} for entity in entities if entity["id_e"] not in existing_same_connectors]
        if len(entities) == 0:
          if scope == 'all':
            print(f"info : toutes les entités disposent déjà du connecteur {lib_connector}")
        else:
           print(f"warning : {len(entities)} n'ont pas encore le connecteur {lib_connector}")
        if action == 'status':
          tirets = "--------------------------------------------------------------------"
          #if input("Voulez-vous afficher le détail? (y/n): ").lower().strip()[:1] == "y":
          if details:
            if len(existing_same_connectors) > 0:
              print(f"info: entités déjà pourvues : \n{existing_same_connectors}\n{tirets}")
            if len(doublons) > 0:
              print(f"info: Doublons : \n{doublons}\n{tirets}")
            if len(entities) > 0:
              print(f"info: Entités sans le connecteur : \n{[e['id_e'] for e in entities]}\n{tirets}")

      elif action in ['update', 'delete']:
        #Filter entities to include only entities with this connector
        entities = [{"id_e": ide} for ide in existing_same_connectors]


    #Sort entities by id_e ASC
    entities = sorted(entities, key=lambda x:int(x['id_e']))

    for entity in entities:
      id_e = entity["id_e"]
      #export id_e to os environnement to use it in connector template
      os.environ["ID_E"] = id_e
      #Check if connector libelle exists for this entity
      if id_e in connectors.keys() and lib_connector in connectors[id_e].keys():
        id_ce_list = connectors[id_e][lib_connector]
        # Its possible have many connectors with the same libelle
        if len(id_ce_list) > 1:
          print(f"warning : des doublons existent pour le connecteur {lib_connector} de l'entité {id_e}")
        # get only the first id_ce
        id_ce = id_ce_list[0]
        if action in ['append', 'update']:
          print(f"info : entité : {id_e} : mise à jour du connecteur : {id_ce}")
          parameters = variables_substitution(connector_template["parameters"])
          result = Connector(session).update(id_e, id_ce, parameters)
          if result and flux_associe:
            Association(session).create(id_e, flux_associe, id_ce, type_connector)
          if result.success and scheduler:
            Connector(session).action(id_e, id_ce, "purge-async")

        elif action == 'dissociate':
          #Dissociate only with the flux specified
          deleteAssociations(session, id_e, id_ce, flux_associe)
        elif action == 'delete':
          deleteConnector(session, id_e, id_ce)
        elif action == 'create':
          # Does not occurs because entities are filtered for this connector
          print(f"warning : le connecteur existe déjà : {id_ce} pour l'entité : {id_e}")
      else:
        if action in ['create', 'append']:
          #Only create connector if does not exists yet
          print(f"info : création du connecteur pour l'entité : {id_e}")
          createConnecteur(session, id_e, connector_template, flux_associe, lib_connector, type_connector, scheduler)
        elif action in ['delete', 'update']:
          # Does not occurs because entities are filtered for this connector
          print(f"warning: Aucun connecteur de type {type_connector}:{id_connector} et libellé {lib_connector} trouvé pour l'entité ({id_e})")


def deleteConnector(session, id_e, id_ce):
  #delete associations for this connector
  deleteAssociations(session, id_e, id_ce)
  # then delete connector
  Connector(session).delete(id_e, id_ce)


def deleteAssociations(session, id_e, id_ce, flux=None):
  #Get all Flux/connector associations for the connector with this id_ce and optional flux if specified
  request = Association(session).getByEntity(id_e, id_ce, flux)
  if request.success:
    associations = request.result
  else:
    print(f"error : {request.result}")
  #Delete all association for this connector
  for association in associations:
    if flux:
      print(f"info : désassociation du connecteur {id_ce} avec le flux {association['flux']}")
    else:
      print(f"info: désassociation du connecteur {id_ce} avec tous les flux")
    Association(session).delete(id_e, association["id_fe"])


def createConnecteur(session, id_e, connector_template, flux_associe, lib_connector, type_connector, scheduler):
  definition = variables_substitution(connector_template["definition"])
  parameters = variables_substitution(connector_template["parameters"])
  #Etape 1 Creation du connecteur sans sa configuration pour l'IDE sélectionnée
  request = Connector(session).create(id_e, definition)
  if request.success:
    id_ce = request.result
  else:
    print(f"error : {request.result}")

  if id_ce:
    # ETAPE 2: configuration du connecteur précédemment créé avec injection des paramètres
    request = Connector(session).update(id_e, id_ce, parameters)
    if request.success and flux_associe:
      Association(session).create(id_e, flux_associe, id_ce, type_connector)
    if request.success and scheduler:
      Connector(session).action(id_e, id_ce, "purge-async")

  else:
    print(f"error : Création du connecteur {lib_connector} KO erreur:{request.result}")

def script_wrapper():
  xstart = datetime.now()
  parser = argparse.ArgumentParser()
  parser.add_argument("-e", "--env", type=str, help="Sélection de l'environnement", choices=['preprod','prod'])
  parser.add_argument("-a", "--action", type=str, help="Sélection de l'action", choices=['create','update', 'append', 'dissociate', 'delete', 'status'])
  parser.add_argument("-o", "--org", type=str, help="Sélection de l'entité")
  parser.add_argument("-f", "--flux", type=str, help="Sélection du flux (type de dossier)")
  parser.add_argument("-s", "--source", type=str, help="fichier json contenant la définition du connecteur ou la liste des connecteurs à supprimer")
  parser.add_argument("-l", "--libelle", type=str, help="Libellé du connecteur")
  parser.add_argument("-t", "--type", type=str, help="type de connecteur (eg. GED)")
  parser.add_argument("-i", "--id_connector", type=str, help="id_connecteur (eg. depot-pastell)")
  parser.add_argument("-d", "--details", action="store_true", help="A utiliser avec l'action status. Pour afficher les id_e des entités concernées")
  parser.add_argument("-r", "--recursive", action="store_true", help="Sélection de l'environnement")
  parser.add_argument("-c", "--scheduler", action="store_true", help="Launch connecteur with existing asynchrone scheduler")
  args = parser.parse_args()

  connector(env=args.env, action=args.action, org=args.org, flux_associe=args.flux, source=args.source,
    type_connector=args.type, id_connector=args.id_connector, lib_connector=args.libelle,
    details=args.details, recursive=args.recursive, scheduler=args.scheduler)

  xend = datetime.now()
  delta = xend - xstart
  msg = f"info : script exécuté en {delta.seconds // 60} minutes et {delta.seconds % 60} secondes"
  print(msg)


def connector(env, action, org=None, flux_associe=None, source=None, type_connector=None, id_connector=None, lib_connector=None, details=None, recursive=None, scheduler=None):
  #Get parameters from config file ~/.pastell-admin
  config = getConfiguration()

  delete_list = None
  connector_template = None

  #Export var to OS environnement
  for var, value in config["var"].items():
    os.environ[var] = value

  #Check for mandatory parameters
  assert action in ['status', 'create', 'append', 'update', 'delete', 'dissociate']
  #if create or append then source is mandatory
  if action in ['create', 'append', 'update']:
    assert source != None
    # Opening JSON file
    f = open(source)
    # returns JSON object as dictionary
    connector_template = json.load(f)
    # Récupération des propriétés du connecteur & substitution des variables
    assert all( key in connector_template.keys() for key in ["definition", "parameters"])
    assert all(key in connector_template["definition"].keys() for key in ["libelle", "id_connecteur", "type"])
    lib_connector = connector_template["definition"]["libelle"]
    id_connector = connector_template["definition"]["id_connecteur"]
    type_connector = connector_template["definition"]["type"]
  #if delete
  elif action == 'delete':
    assert (lib_connector and type_connector and id_connector) or source
    if source:
      f = open(source)
      # returns JSON object as dictionary
      delete_list = json.load(f)
      # Chek if All items in delete_list contains keys id_ce and id_e
      assert all(key in item.keys() for item in delete_list for key in ["id_ce", "id_e"])
  elif action == 'dissociate':
    assert (lib_connector and type_connector and id_connector and flux_associe)
  elif action == 'status':
    assert (lib_connector)


  server = config[env]['server']
  user = config[env]['login']
  pwd = config[env]['password']
  #Session used by all the script
  session = Session(server,  user, pwd)
  #Check if session is valid
  if not session.valid:
    exit()
  if action in ['status', 'create', 'append', 'dissociate', 'delete', 'update']:
    scope = 'all'
    if org:
      scope = org
    taskConnectorFor(session, scope, action, connector_template, delete_list, flux_associe, lib_connector, type_connector, id_connector, details, recursive, scheduler)


if __name__ == '__main__':
  script_wrapper()






#!/usr/bin/env python3
import csv
from datetime import datetime
import argparse
from .api.client import Session, Association, Entity
from .shared.configuration import getConfiguration

def writeResults(csv_list, output, fields):
  with open(output, 'w', encoding='UTF8', newline='') as f:
    writer = csv.writer(f)
    # write the header
    writer.writerow(fields)
    # write multiple rows
    writer.writerows(csv_list)

def script_wrapper():
  xstart = datetime.now()
  parser = argparse.ArgumentParser()
  parser.add_argument("-e", "--env", type=str, help="Sélection de l'environnement", choices=['preprod','prod'], required=True)
  parser.add_argument("-o", "--org", help="id_e de l'entité")
  parser.add_argument("-c", "--csv", help="Fichier csv en sortie. /tmp/associations.csv par défaut")
  args = parser.parse_args()

  associations(env=args.env, org=args.org, csv=args.csv)

  xend = datetime.now()
  delta = xend - xstart
  msg = f"info : script exécuté en {delta.seconds // 60} minutes et {delta.seconds % 60} secondes"
  print(msg)


def associations(env, org=None, csv="/tmp/associations" ):
  #Get parameters from config file ~/.pastell-admin
  config = getConfiguration()

  server = config[env]['server']
  user = config[env]['login']
  pwd = config[env]['password']
  session = Session(server,  user, pwd)
  #START PROCESS
  fields = ['id_fe', 'id_e', 'flux', 'id_ce', 'type']
  csv_list = []
  #Get stats from Pastell API
  entites = False
  if org:
    entites = [{"id_e": org}]
  else:
    entites = Entity(session).getAll().result
  if entites:
    # Boucle sur les entités
    for entite in entites:
      id_e = entite['id_e']
      #Get all associations
      associations = Association(session).getByEntity(id_e).result
      #url = f"https://{server}/api/v2/entite/{id_e}/flux/"
      #request = requests.get(url, auth=HTTPBasicAuth(user, pwd))
      #associations = json.loads(request.text)
      for association in associations:
        data = []
        for field in fields:
          data.append(association[field])
        csv_list.append(data)

    writeResults(csv_list, csv, fields)

if __name__ == '__main__':
  script_wrapper()










#!/usr/bin/env python3
import csv, argparse
from datetime import datetime
from .api.client import Session, Entity
from .shared.configuration import getConfiguration

# USAGE
# This script list pastell entities with sub entities.

def writeResults(OUTPUT, csv_list, ACTIVE):
  header = ['id_e', 'denomination', 'siren', 'type', 'centre_de_gestion', 'entite_mere']
  if ACTIVE:
    header.append('is_active')

  with open(OUTPUT, 'w', encoding='UTF8', newline='') as f:
    writer = csv.writer(f)
    # write the header
    writer.writerow(header)
    # write multiple rows
    writer.writerows(csv_list)

def script_wrapper():
  xstart = datetime.now()
  parser = argparse.ArgumentParser()
  parser.add_argument("-e", "--env", type=str, help="Sélection de l'environnement", choices=['preprod','prod'], required=True)
  parser.add_argument("-c", "--csv", help="Fichier csv en sortie. /tmp/organismes.csv par défaut")
  parser.add_argument("-a", "--active", help="Retourne l'information active", action="store_true")
  args = parser.parse_args()

  org(env=args.env, csv=args.csv, active=args.active)

  xend = datetime.now()
  delta = xend - xstart
  msg = f"info : script exécuté en {delta.seconds // 60} minutes et {delta.seconds % 60} secondes"
  print(msg)


def org(env, csv="/tmp/organismes.csv", active=False):
  #Get parameters from config file ~/.pastell-admin
  config = getConfiguration()

  ENV = env
  OUTPUT = csv
  ACTIVE = active
  SERVER = config[ENV]['server']
  user = config[ENV]['login']
  pwd = config[ENV]['password']
  #START PROCESS
  session = Session(SERVER,  user, pwd)
  #Check if session is valid
  if not session.valid:
    exit()

  csv_list = []
  #Get Entities from Pastell API
  if ACTIVE:
    entities = Entity(session).getAllWithStatus()
  else:
    entities = Entity(session).getAll()
  if entities.success:
    if ACTIVE:
      entites_json = entities.result["all"]
    else:
      entites_json = entities.result
    # Boucle sur les entités
    for entite in entites_json:
      id_e = entite['id_e']
      denomination = entite['denomination']
      siren = entite['siren']
      type = entite['type']
      centre_de_gestion = entite['centre_de_gestion']
      entite_mere = entite['entite_mere']
      #Enregistrement dans la liste csv
      raw = [id_e, denomination, siren, type, centre_de_gestion, entite_mere]
      if ACTIVE:
        raw.append(entite['is_active'])
      csv_list.append(raw)

    writeResults(OUTPUT, csv_list, ACTIVE)

if __name__ == '__main__':
  script_wrapper()








#!/usr/bin/env python3
import requests, json, csv
from requests.auth import HTTPBasicAuth
import os, sys
from datetime import datetime
import getopt, configparser

# USAGE
# This script list pastell documents in a specific state on time interval and export result in json file and a csv file.
# json file is a dict and can be used as inpur with pastell_delete.py script :
# option -e or --env => to select environnement execution (prod or preprod). default is preprod
# option -s or --status => pastell state to filter. eg : terminate#
# option -t or --type => Type de dossier. Exemple autres-studio-sans-tdt
#
# ./delete_any.py --env=preprod --status=terminate --type=autres-studio-sans-tdt
#
#

options, remainder = getopt.getopt(
      sys.argv[1:],
      'e:s:t:',
      ['env=','status=','type='])

xstart = datetime.now()

WKD = os.path.abspath(os.path.dirname(__file__))
ENV = 'preprod'
OUTPUT = "result"
LIMIT = 0
CLEAR = False
STATUS = "terminate"
TYPE = "autres-studio-sans-tdt"
START = "2015-01-01"
END = xstart.strftime("%Y-%m-%d")
FILE_SUCCESS = '%s/success' % WKD
FILE_TMP = '%s/tmp' % WKD
FILE_ERROR = '%s/errors' % WKD

config = configparser.ConfigParser()
config.read("%s/config.ini" % WKD)


for opt, arg in options:
  if opt in ('e', '--env'):
    ENV = arg
  elif opt in ('s', '--status'):
    STATUS = arg
  elif opt in ('t', '--type'):
    TYPE = arg


SERVER = config[ENV]['server']
user = config[ENV]['login']
pwd = config[ENV]['password']

FINISHED = False
while not FINISHED:
  url = "https://%s/api/v2/entite/1/document/?type=%s&last_action=%s" % (SERVER, TYPE, STATUS)
  request0 = requests.get(url, auth=HTTPBasicAuth(user, pwd))
  data = json.loads(request0.text)
  nb = len(data)
  print("Supression de %s documents" % str(nb))
  if nb == 0:
    FINISHED = True
  for doc in data:
    url2 = "https://%s/api/v2/entite/%s/document/%s/action/supression" % (SERVER, doc['id_e'], doc["id_d"])
    request = requests.post(url2, auth=HTTPBasicAuth(user, pwd))
    print(request.status_code)
    print(request.text)

xend = datetime.now()
print (str(xend))
print ("script execute en : " + str((xend-xstart).total_seconds()))



#!/usr/bin/env python3
from itertools import count
import json, csv
import os
from datetime import datetime
import argparse
from .api.client import Session, Stat, Document
from .shared.configuration import getConfiguration

# USAGE
# This script list pastell documents in a specific state on time interval and export result in json file and a csv file.
# json file is a dict and can be used as inpur with pastell_delete.py script :
# option -e or --env => to select environnement execution (prod or preprod). default is preprod
# option -s or --status => pastell states to filter. eg : tdt-error,termine
# option -f or --flux => pastell flux to filter. eg : actes-generiques or actes-generique,deliberations-studio
# option -o or --output => define a name for 2 output files (csv + json). Default is result
# option -g or -- org => Filter to only one id_e
# option -d or --date => Filter to one date
# option -a or --start => Filter to date begin
# option -b or --end => Filter to date end
# option -m or --merge => to merge results in existing output file
# option -t or --transit => use transit status an not last status
# option -n or --count => to only count docs without details
# option -l or --list => export id documents as list
# option -j or --json => export result as json file
#

def summaryMessage(status, transit, flux, org, start, end, output):
  message = ["API PASTELL : Recherche des documents dont :"]
  if status:
    if not transit:
      message.append("le statut est : %s" % ", ".join(status))
    else:
      message.append("le statut est passé par : %s" % ", ".join(status))
  if flux:
    message.append("pour les flux : %s" % ", ".join(flux))
  else:
    message.append("pour tous les types de flux")
  if org:
    message.append("pour l'entité : %s" % org)
  else:
    message.append("pour toutes les entités")
  if start and end:
    message.append("entre le %s et le %s" % (start, end))
  elif start:
    message.append("depuis le %s" % start)
  elif end:
    message.append("antérieurs à %s" % end)
  if count:
    message.append("Analyse : dénombrement des dossiers concernés par flux et entité")
  elif list:
    message.append("Analyse : listing des dossiers concernés par flux et entité")
  if json:
    message.append("le fichier %s.json sera créé en sortie" % output)
  message.append("le fichier %s.csv sera créé en sortie" % output)
  print("\n".join(message))

def getStats(session, org=None):
  stats = False
  #Get stats from Pastell API. If ORG is None get all stats
  request = Stat(session).get(id_e=org)
  if request.success:
    stats = request.result
  return stats

def getDocs(session, entite, dossier, status, start, end, csv_list, json_list, err_list, transit=False, count=False, list=False):
  interval = start + " - " + end
  etat_msg = "l'état est"
  if transit:
    etat_msg += " passé par"
  print("Récupérations des documents du flux %s dont %s %s pour l'entité (%s)" % (dossier, etat_msg, status, entite["id_e"]))

  request = Document(session).getByFilter(entite["id_e"], dossier, status, start, end, transit)
  if request.success:
    docs_json = request.result
    if count:
      csv_list.append([interval, entite['id_e'], entite['denomination'], entite['siren'], dossier, status, len(docs_json)])
      json_list.append({"id_e" : entite['id_e'], "dossier": dossier, "documents": len(docs_json), "status": status })
    elif list:
      for document in docs_json:
        csv_list.append([interval, entite['id_e'], entite['denomination'], entite['siren'],
          dossier, status, document["id_d"], document["last_action_date"]])
        json_list.append({"id_e" : entite['id_e'], "dossier": dossier, "document": document["id_d"], "status": status })
    else:
      print("warning: Rien à faire choisir entre --count ou --list")
  else:
    err_list.append(entite['id_e'])


def writeResults(csv_file, csv_list, json_file, json_list, list, merge):
  header = ['interval', 'id', 'denomination', 'siren', 'dossier', 'statut', 'documents']
  if list:
    header.append('date')

  with open(csv_file, 'a', encoding='UTF8', newline='') as f:
    writer = csv.writer(f)
    # write the header
    if not merge:
      writer.writerow(header)
    # write multiple rows
    writer.writerows(csv_list)

  if json_file:
    with open(json_file, 'w', encoding='UTF8') as f:
      json.dump({ "docs" : json_list }, f)


def script_wrapper():
  xstart = datetime.now()
  parser = argparse.ArgumentParser()
  parser.add_argument("-e", "--env", type=str, help="Sélection de l'environnement", choices=['preprod','prod'], required=True)
  parser.add_argument("-s", "--status", type=str, nargs="*", help="Etats à filtrer", required=True)
  parser.add_argument("-f", "--flux", type=str, nargs="*", default=[], help="Sélection du flux (type de dossier)")
  parser.add_argument("-o", "--output", type=str, help="Nom des fichiers exportés (csv + json)")
  parser.add_argument("-g", "--org", type=str, help="Sélection de l'entité")
  #parser.add_argument("-d", "--date", type=str, help="Date des documents")
  parser.add_argument("-a", "--start", type=str, help="Date de début")
  parser.add_argument("-b", "--end", type=str, help="Date de fin")
  parser.add_argument("-n", "--count", action="store_true", help="Exporte le nombre de documents concernés")
  parser.add_argument("-l", "--list", action="store_true", help="Exporte tous les id_d des documents")
  parser.add_argument("-j", "--json", action="store_true", help="Exporte en plus le résultat dans un fichier json")
  parser.add_argument("-m", "--merge", action="store_true", help="Merge les résultats dans un fichier existant")
  parser.add_argument("-t", "--transit", action="store_true", help="Utilise passé par l'état plutôt que le dernier état")
  args = parser.parse_args()
  #Print summary script message witl all options values
  summaryMessage(args.status, args.transit, args.flux, args.org, args.start, args.end, args.output)
  #Call main method with parameters
  find(env=args.env, org=args.org, output=args.output, flux=args.flux, status=args.status,
    start=args.start, end=args.end, count=args.count, list=args.list, json=args.json,
    merge=args.merge, transit=args.transit)

  xend = datetime.now()
  delta = xend - xstart
  msg = f"info : script exécuté en {delta.seconds // 60} minutes et {delta.seconds % 60} secondes"
  print(msg)

def find(env, org=None, output="result", flux=None, status=None, start="2015-01-01",
  end=datetime.now().strftime("%Y-%m-%d"), count=True, list=False, json=False, merge=False, transit=False):
 #Get parameters from config file ~/.pastell-admin
  config = getConfiguration()

  json_file = '%s.json' % (output)
  csv_file = '%s.csv' % (output)

  if os.path.isfile(csv_file) and not merge:
    os.remove(csv_file)
  if os.path.isfile(json_file):
    os.remove(json_file)

  server = config[env]['server']
  user = config[env]['login']
  pwd = config[env]['password']

  session = Session(server,  user, pwd)
  #Check if session is valid
  if not session.valid:
    exit()

  csv_list = [] # used for csv file output
  json_list = [] # used for json file output
  err_list = [] # used for static tmp error file

  #Get stats from Pastell API
  stats_entites_json = getStats(session, org)
  if stats_entites_json:
    # Boucle sur les entités
    for index in stats_entites_json:
      stats = stats_entites_json[index]
      #Boucle sur tous les types de flux
      for dossier in stats['flux']:
        # Filtre optionnel sur les flux
        if dossier in flux or not flux:
          #Boucle sur les statuts
          for _status in status:
            if _status in stats['flux'][dossier]:
              #Récupération individuelle de tous les documents concernés avec pagination (1000 documents)
              getDocs(session, stats['info'], dossier, _status, start, end, csv_list, json_list, err_list, transit, count, list)

    writeResults(csv_file, csv_list, json_file, json_list, list, merge)

if __name__ == '__main__':
  script_wrapper()




#!/usr/bin/env python3
import csv
from datetime import datetime
import argparse
from .api.client import Session, Stat
from .shared.configuration import getConfiguration

"""
Ce script permet de générer un fichier csv de statistiques

  Args:
    option -e or --env => to select environnement execution (prod or preprod). default is preprod
    option -o or --org => pastell entity to filter. eg : 1. Optional
    option -c or --csv => Chemin et nom du fichier csv généré. Optional
    option -i or --infos => pour afficher le statut des organismes et  notamment si l'entité est active. Par défault, affiche les statistiques sur les flux.


  Usage :
    ./pastell_stats.py --env=preprod --org=1 --csv=info.csv --infos
    ./pastell_stats.py --env=preprod  --csv=/tmp/statistiques_flux.csv

"""

def writeResults(OUTPUT, csv_list, csv_headers):

  with open(OUTPUT, 'w', encoding='UTF8', newline='') as f:
    writer = csv.writer(f)
    # write the header
    writer.writerow(csv_headers)
    # write multiple rows
    writer.writerows(csv_list)

def script_wrapper():
  xstart = datetime.now()
  parser = argparse.ArgumentParser()
  parser.add_argument("-e", "--env", type=str, help="Sélection de l'environnement", choices=['preprod','prod'], required=True)
  parser.add_argument("-o", "--org", help="id_e de l'entité")
  parser.add_argument("-f", "--flux", help="type de flux")
  parser.add_argument("-c", "--csv", help="Fichier csv en sortie. /tmp/stats-YYYY-MM-DD.csv par défaut")
  parser.add_argument("-i", "--infos", help="pour afficher le statut des organismes et  notamment si l'entité est active. Par défault, affiche les statistiques sur les flux", action="store_true")
  args = parser.parse_args()

  stats(env=args.env, org=args.org, csv=args.csv, infos=args.infos, flux=args.flux)

  xend = datetime.now()
  delta = xend - xstart
  msg = f"info : script exécuté en {delta.seconds // 60} minutes et {delta.seconds % 60} secondes"
  print(msg)

def stats(env, org=None, csv=None, infos=None, flux=None):
  #Get parameters from config file ~/.pastell-admin
  config = getConfiguration()

  ladate = datetime.now()
  xdate = ladate.strftime("%Y-%m-%d")
  ENV = env
  OUTPUT = csv or "/tmp/stats-%s.csv" % xdate
  ORG = org
  INFOS = infos
  TYPE = flux

  SERVER = config[ENV]['server']
  user = config[ENV]['login']
  pwd = config[ENV]['password']

  csv_headers = ['date', 'id_e', 'flux', 'statut', 'nombre']
  if INFOS:
    csv_headers = ['id_e', 'type', 'denomination', 'siren', 'date_inscription', 'etat', 'entite_mere', 'centre_de_gestion', 'is_active']
  #START PROCESS
  session = Session(SERVER,  user, pwd)
  #Check if session is valid
  if not session.valid:
    exit()
  csv_list = []
  #Get stats from Pastell API
  stats = Stat(session).get(id_e=ORG, type=TYPE)
  if stats.success:
    stats_entites_json = stats.result
    if stats_entites_json:
      # Boucle sur les entités
      for index in stats_entites_json:
        stats = stats_entites_json[index]
        infos = stats['info']
        if INFOS:
          raw_data = []
          for prop in csv_headers:
            raw_data.append(infos[prop])
          csv_list.append(raw_data)
        else:
          id_e = infos['id_e']
          #Boucle sur tous les types de flux
          for dossier in stats['flux']:
            #Boucle sur les statuts
            for status in stats['flux'][dossier]:
              total_docs = int(stats['flux'][dossier][status])
              #Enregistrement dans la liste csv
              csv_list.append([xdate, id_e, dossier, status, total_docs])

      writeResults(OUTPUT, csv_list, csv_headers)

    else:
      print(stats.result)

if __name__ == '__main__':
  script_wrapper()








#!/usr/bin/env python3
from ast import Assert, Param
from distutils.log import error
import requests, json
from requests.auth import HTTPBasicAuth
from urllib.parse import urlencode
from datetime import datetime

class Session():
    """
    Classe permettant d'instancier les informations de session
    Serveur + Authentification
    """
    def __init__(self, server, user, password):
        """
        Usage :
            from pastell.api import Session
            session = Session(url_pastell, user, password)
            if session.valid:
                ...

        Args:
            server ( str ): URL du serveur Pastell
            user ( str ) : user du compte Pastell à utiliser
            password ( str ) : Mot de passe associé
        """
        self.server = server
        self.auth = HTTPBasicAuth(user, password)
        #Check if session is valid
        version = Version(self).get()
        self.valid =  version.success == True
        if self.valid:
            print(f" ------------------------\nPastell version {version.result['version']} \n ------------------------")
        else:
            print(version.result)



class PastellSession():
    """
    Classe permettant de stocker les informations de session
    Serveur + Authentification
    Cette classe est ensuite étendue par toutes les classes suivantes
    ce qui permet de partager les propriétés server et auth
    """
    def __init__(self, session):
        self.server = session.server
        self.auth = session.auth

class Result():

    def __init__(self, success, result):
        self.success = success
        self.result = result

class Version(PastellSession):

    def __init__(self, session):
        super().__init__(session)

    def get(self):
        url = f"https://{self.server}/api/v2/version"
        request = requests.get(url, auth=self.auth)
        if request.status_code == 200:
            result = json.loads(request.text)
            success = True
        else:
            result = request.text
            success = False

        return Result(success, result)

class Stat(PastellSession):

    def __init__(self, session):
        super().__init__(session)

    def get(self, id_e=None, type=None):
        url = f"https://{self.server}/api/v2/document/count"
        if id_e or type:
            params = {}
            if id_e:
                params["id_e"] = id_e
            if type:
                params["type"] = type
            url = f"{url}?{urlencode(params)}"
        request = requests.get(url, auth=self.auth)
        if request.status_code == 200:
            result = json.loads(request.text)
            success = True
        else:
            result = request.text
            success = False

        return Result(success, result)


class Entity(PastellSession):

    def __init__(self, session):
        super().__init__(session)

    def getAllWithStatus(self):
        """
        Méthode permettant de retourner tous les organismes actifs(entités) Pastell

        Returns:
             dict: with 2 keys success (bool) and result (dict with 3 lists)
        """
        stats = Stat(self).get()
        success = False
        if stats.success:
            active = []
            no_active = []
            for ide, obj in stats.result.items():
                if obj["info"]["is_active"] == "1":
                    active.append(obj["info"])
                else:
                    no_active.append(obj["info"])
            print(f"info: {len(no_active)} entités sont inactives et {len(active)} entités sont actives")
            succcess = True
            result = {'all': [*active, *no_active],'active': active, 'no_active': no_active}
        else:
            result = stats.result
        return Result(True, result)



    def getAll(self):
        """
        Méthode permettant de retourner tous les organismes (entités) Pastell

        Returns:
             dict: with 2 keys success (bool) and result (list)
        """

        url = f"https://{self.server}/api/v2/entite"
        request = requests.get(url, auth=self.auth)

        if request.status_code == 200:
            result = json.loads(request.text)
            success = True
        else:
            result = request.text
            success = False

        return Result(success, result)

    def getFilles(self, id_e, tree=None):
        """
        Méthode permettant de retourner tous les organismes filles (entités/services)  d'une entité Pastell

        Returns:
             dict: with 2 keys success (bool) and result (list)
        """
    #Interdit pour entité racine 0
        assert int(id_e) > 0

        entities = [id_e]
        work = [id_e]
        level = 1
        calls = 0
        erreur = False
        while len(work) > 0 and not erreur:
            newLevel = []
            finished = []
            for ide in work:
                request = Entity(self).detail(ide)
                if request.success:
                    calls +=1
                    filles = []
                    if len(request.result["entite_fille"]) > 0:
                        filles = [e["id_e"] for e in request.result["entite_fille"]]
                        entities.extend(filles)
                        newLevel.extend(filles)
                    #print (f"level : {level} : call {calls} : {ide} --> {filles}")
                    finished.append(ide)
                else:
                    erreur = True
                    result = request.result
                    break
            if not erreur:
                for id in finished:
                    work.remove(id)
                work.extend(newLevel)
                level += 1
        entities.sort(key = int)
        result = [{"id_e": entity} for entity in entities]
        return Result(not erreur, result)









    def detail(self, id_e):
        """
        Méthode permettant de retourner le détail d'un organisme donné (fourni par id_e)

        Args:
            id_e ( str ): identifiant de l'entité.

        Returns:
            dict: with 2 keys success (bool) and result (dict)
        """
        assert isinstance(id_e, str)
        url = f"https://{self.server}/api/v2/entite/{id_e}"
        request = requests.get(url, auth=self.auth)
        if request.status_code == 200:
            result = json.loads(request.text)
            success = True
        else:
            result = request.text
            success = False

        return Result(success, result)

class Connector(PastellSession):

    def __init__(self, session):
        super().__init__(session)

    def __addToDict(self, item, dictionary):
        if not item["id_e"] in dictionary.keys():
            dictionary[item["id_e"]] = { item["libelle"] : [item["id_ce"]] }
        else:
            if not item["libelle"] in dictionary[item["id_e"]]:
                dictionary[item["id_e"]][item["libelle"]] = [item["id_ce"]]
            else:
                dictionary[item["id_e"]][item["libelle"]].append(item["id_ce"])

    def getAll(self, scope, id_connecteur=None, outputFormat='list'):
        """
        Get all connectors with one id_connecteur (option) (eg 'depot-pastell')
        for one id_e or all organisms

        Args:
            scope (str): 'all' or id_e
            id_connecteur (str): id_connecteur eg "depot-pastell"
            outputFormat (str): 'list' or 'dict'. Defaults = 'list'

        Returns:
            dict: with 2 keys success (bool) and result (dict)
            result dic structure item :{id_e:{"lib_connector":[id_ce]}} or
            result list structure = raw response from Pastell
            sample result dict : {"123":{"myfirst_connector":["47"]}, {"mysecond_connector":["48"]},"124":{...}}
        """
        url = f"https://{self.server}/api/v2/entite/{scope}/connecteur"
        if scope == 'all':
            url = f"https://{self.server}/api/v2/connecteur/{scope}/"
            if id_connecteur:
                url = f"https://{self.server}/api/v2/connecteur/{scope}/{id_connecteur}"

        request = requests.get(url, auth=self.auth)
        if request.status_code == 200:
            success = True
            connectors = json.loads(request.text)
            if outputFormat == 'list':
                if not id_connecteur:
                    #raw response from PAstell
                    result = connectors
                else:
                    #raw response from PAstell filtered by id_connecteur
                    result = [connector for connector in connectors if connector["id_connecteur"] == id_connecteur]
            elif outputFormat == 'dict':
                #create dict id_e:libelle:[id_ce]
                result = {}
                for connector in connectors:
                    if not id_connecteur or (id_connecteur and connector["id_connecteur"] == id_connecteur):
                        self.__addToDict(connector, result)

        else:
            success = False
            result = request.text

        return Result(success, result)

    def delete(self, id_e, id_ce):
        url = f"https://{self.server}/api/v2/entite/{id_e}/connecteur/{id_ce}"
        request = requests.delete(url, auth=self.auth)
        result = json.loads(request.text)
        if request.status_code == 200:
            print(f"info: Connecteur: {id_ce} supprimé avec succès")

    def update(self, id_e, id_ce, parameters):
        """_summary_

        Args:
            id_e (_type_): _description_
            id_ce (_type_): _description_
            parameters (_type_): _description_

        Returns:
            _type_: _description_
        """
        urlConnecteur = f"https://{self.server}/api/v2/entite/{id_e}/connecteur/{id_ce}/content/"
        responseConnecteur = requests.patch(urlConnecteur, data=parameters, auth=self.auth)
        success = False
        if responseConnecteur.ok:
            success = True
        else:
            print(f"error: Creation config connecteur {id_ce} KO erreur:{responseConnecteur.text}")

        return Result(success, None)

    def create(self, id_e, definition):
        """
        Creation du connecteur sans sa configuration pour l'IDE sélectionnée

        Args:
            id_e (str): Identifiant de l'entité
            definition (dict): Dictionnaire de clés valeurs

        Returns:
            dict: with 2 keys success (bool) and result (str) id_ce created
        """

        url = f"https://{self.server}/api/v2/entite/{id_e}/connecteur/"
        request = requests.post(url, data=definition, auth=self.auth)
        if request.ok:
            data = json.loads(request.text)
            result = data["id_ce"]
            success = True
        else:
            success = False
            result = request.text

        return Result(success, result)

    def action(self, id_e, id_ce, action):
        url = f"https://{self.server}/api/v2/entite/{id_e}/connecteur/{id_ce}/action/{action}"
        request = requests.post(url, auth=self.auth)
        if request.ok:
            data = json.loads(request.text)
            result = data["last_message"]
            success = data["result"]
        else:
            success = False
            result = request.text

        return Result(success, result)

class Association(PastellSession):

    def __init__(self, session):
        super().__init__(session)


    def getByEntity(self, id_e, id_ce=None, flux=None):
        """
        Returns list off all flux associations for organism and in option, with the connector identified by his id_ce

        Args:
            id_e (str): Identifiant de l'organisme
            id_ce (str, optional): Identifiant du connecteur. Defaults to None.
            flux (_type_, optional): Identifiant du flux. Defaults to None.

        Returns:
            dict: with 2 keys success (bool) and result (list) of associations
        """
        url = f"https://{self.server}/api/v2/entite/{id_e}/flux"
        request = requests.get(url, auth=self.auth)
        if request.status_code == 200:
            data = json.loads(request.text)
            if id_ce:
                associations = []
                for association in data:
                    if flux:
                        if association["id_ce"] == id_ce and association["flux"] == flux:
                            associations.append(association)
                    else:
                        if association["id_ce"] == id_ce:
                            associations.append(association)
            else:
                associations = data

            success = True
            result = associations
        else:
            result = request.text
            success = False

        return Result(success, result)

    def delete(self, id_e, id_fe):
        url = f"https://{self.server}/api/v2/entite/{id_e}/flux?id_fe={id_fe}"
        request = requests.delete(url, auth=self.auth)
        result = json.loads(request.text)
        if request.status_code == 200:
            print(f"info: Suppression de l'association {id_fe} pour l'entité ({id_e} effectuée)")

    def create(self, id_e, flux, id_ce, type):
        """
        Méthode permettant d'associer un connecteur à un flux

        Args:
            id_e (str): Identifiant de l'organisme
            flux (str): Identifiant du flux
            id_ce (str): Identifiant du connecteur
            type (str): type de connecteur (GED...)
        """
        data = {"type": type}
        # Association avec le flux fournis en paramètre
        url = f"https://{self.server}/api/v2/entite/{id_e}/flux/{flux}/connecteur/{id_ce}/"
        response = requests.post(url, data=data, auth=self.auth)
        if response.ok:
            print(f"info: Association connecteur {id_ce} ok type_flux:{flux}, id_e:{id_e}")
        else:
            print(f"error: Association connecteur {id_ce} KO erreur:{response.text}")


class Document(PastellSession):

    def __init__(self, session):
        super().__init__(session)


    def getByFilter(self, id_e, dossier=None, status=None, start=None, end=None, transit=False, pagination=1000 ):
        #check types
        date_format = "%Y-%m-%d"
        for date in (start, end):
            if date:
                assert datetime.strptime(date, date_format)
        assert type(pagination) == int
        assert type(transit) == bool
        success = None
        # On positionne l'offset à 0 pour la première requête
        offset = 0
        completed = False
        parameters = {}
        result = []
        #Construct URL parameters
        if dossier:
            parameters["type"] = dossier
        if status and transit:
            parameters["etatTransit"] = status
            if start:
                parameters["state_begin"] = start
            if end:
                parameters["state_end"] = end
        elif status and not transit:
            parameters["lastetat"] = status
            if start:
                parameters["last_state_begin"] = start
            if end:
                parameters["last_state_end"] = end

        while not completed:
            pagination_parameters = {
                'limit': pagination,
                'offset': offset
            }
            url = f"https://{self.server}/api/v2/entite/{id_e}/document"
            request = requests.get(url, params={**parameters, **pagination_parameters}, auth=self.auth)
            if request.status_code == 200:
                docs_json = json.loads(request.text)
                if len(docs_json) > 0:
                    # On augmente l'offset de la limite pour une éventuelle prochaine requête
                    offset += pagination
                    result.extend(docs_json)
                else:
                    # 0 enregistrement donc on a tout récupérer
                    completed = True
                    success = True
                if len(docs_json) < pagination:
                    completed = True
                    success = True
            else:
                result = f"erreur de récupération des documents pour l'entité {id_e}"
                success = False
                break

        return Result(success, result)








